import mmc.*;
import java.io.InputStreamReader;
import java.io.LineNumberReader;
import java.util.StringTokenizer;

public class Amanda{

    private final static double LENGTH=720;
    private final static double RADIUS=180;
    private final static double DEPTH=1730;
    private final static double BIG=-6.4e6;
    private static boolean USER=false;
    private static boolean SDEC=false;
    private static String type;

    public static void main(String[] args){
	double length=LENGTH, radius=RADIUS, depth=DEPTH;
	String med="Ice";

	for(int n=0; n<args.length; n++){
	    if(args[n].equals("-help")){
		System.out.println("\n"+
"This program propagates atmospheric muons through the "+med+" to the detector\n"+
"Available options are: -length=[LENGTH of the volume of the detector in meters]\n"+
"                       -radius=[RADIUS of the volume of the detector in meters]\n"+
"                       -depth=[DEPTH to the center of the detector in meters]\n"+
"                       -sdec  enable stopped muon decay treatment\n"+
"                       -user  enable the mpmcwj-energy user line\n");
		return;
	    }
	    else if(args[n].equals("-user")){
		USER=true;
	    }
	    else if(args[n].equals("-sdec")){
		SDEC=true;
	    }
	    else if(args[n].startsWith("-length=")){
		try{
		    length=Double.parseDouble(args[n].substring(8));
		}catch(Exception error){
		    length=0;
		}
	    }
	    else if(args[n].startsWith("-radius=")){
		try{
		    radius=Double.parseDouble(args[n].substring(8));
		}catch(Exception error){
		    radius=0;
		}
	    }
	    else if(args[n].startsWith("-depth=")){
		try{
		    depth=Double.parseDouble(args[n].substring(7));
		}catch(Exception error){
		    depth=0;
		}
	    }
	}

	System.err.println(Output.version);
	if(length<=0){
	    System.err.println("Warning: length is not a valid number");
	    length=LENGTH;
	}
	if(radius<=0){
	    System.err.println("Warning: radius is not a valid number");
	    radius=RADIUS;
	}
	if(depth<=0){
	    System.err.println("Warning: depth is not a valid number");
	    depth=DEPTH;
	}

	System.err.println("length = "+Output.f(length)+" m,  radius = "+Output.f(radius)+" m,  depth = "+Output.f(depth)+" m");

	Propagate p1 = new Propagate(med, -1.0, 0.01);
	Propagate p2 = new Propagate(med, 5.e2, -1.0);
	Propagate p3 = new Propagate(med, -1.0, -1.0);
	p1.contiCorr=true;  // To randomize the continuous energy losses, use only for small vcut
	p1.exactTime=true;  // To compute local time of the particle exactly
	p2.sdec=SDEC;       // To enable stopped muon decay
	p2.exactTime=true;  // To compute local time of the particle exactly
	p1.interpolate("all", ".amanda");
	p2.interpolate("all", ".amanda");
	p3.interpolate("all", ".amanda");
	System.err.println(" ---  *** Enter your input in F2000 format now ***  --- ");

	try{
	    LineNumberReader file = new LineNumberReader(new InputStreamReader(System.in));
	    String line, taux, hist;
	    StringTokenizer st;

	    int gen, num=0;
	    double b, d, r1, r2, h1, h2;
	    double x, y, z, th, phi, l, e, t, result;

	    double sinth, sinphi, costh, cosphi, e0=0, e1=0, e2=0, n2, nx=0, ny=0, nz=0;
	    double hr1=0, hr2=0, er1=0, er2=0, erc=0, nrx=0, nry=0, nrz=0;

	    long events=0, tracks=0, vertices=0, missed=0;

	    if((line=file.readLine())!=null){
		System.out.println(line);
		System.out.println("HI  "+Output.version+"\nHI  Use Sun java v1.3 or higher: \"java Amanda\"");
		System.out.println("HI  length="+length+" radius="+radius+" depth="+depth+" medium=\""+med+"\"");
		if(USER) System.out.println("HI  User line mpmcwj-energy will be enabled");
		else System.out.println("HI  Use \"-user\" option to enable mpmcwj-energy user line");
	    }
	    while((line=file.readLine())!=null){
		st = new StringTokenizer(line);
		taux=st.nextToken();
		if("TR".equals(taux)){

		    tracks++;
		    gen=Integer.parseInt(st.nextToken())+num;
		    st.nextToken();
		    type=st.nextToken();
		    if("mu+".equals(type)||"mu-".equals(type)){
			x=Double.parseDouble(st.nextToken());
			y=Double.parseDouble(st.nextToken());
			z=Double.parseDouble(st.nextToken());
			th=Double.parseDouble(st.nextToken());
			phi=Double.parseDouble(st.nextToken());
			st.nextToken();
			e=Double.parseDouble(st.nextToken());
			t=Double.parseDouble(st.nextToken());

			b=x*Math.cos(phi*Math.PI/180)+y*Math.sin(phi*Math.PI/180);
			d=b*b+radius*radius-x*x-y*y;
			if(d>0){
			    d=Math.sqrt(d);
			    r1=z-(b-d)/Math.tan(th*Math.PI/180);
			    r2=z-(b+d)/Math.tan(th*Math.PI/180);
			    if(r2>length/2||r1<-length/2){ h1=z; h2=z; }
			    else{ h1=Math.min(r1, length/2); h2=Math.max(r2, -length/2); }
			}
			else{ h1=z; h2=z; }
			if(h1==h2) missed++;

			if(USER){
			    sinth=-Math.cos(th*Math.PI/180); sinphi=-Math.sin(phi*Math.PI/180);
			    costh=Math.sin(th*Math.PI/180); cosphi=-Math.cos(phi*Math.PI/180);
			    n2=x*costh*cosphi+y*costh*sinphi+z*sinth;
			    nx=x-costh*cosphi*n2;
			    ny=y-costh*sinphi*n2;
			    nz=z-sinth*n2;

			    if(nz>h1) flag=1;
			    else if(nz>h2) flag=2;
			    else flag=3;
			}

			hist=""; if(USER){ e1=0; e2=0; ec=0; }
			result=propagateTo(h1, e, nz, 1, p1, 0,
					   t*1.e-9, x*1.e2, y*1.e2, z*1.e2, 180-th, phi<180?phi+180:phi-180);
			if(USER) e1=result; l=p1.p.r;
			if(result>0){
			    result=propagateTo(h2, result, nz, 2, p2, gen,
					       p1.p.t, p1.p.x, p1.p.y, p1.p.z, p1.p.theta, p1.p.phi);
			    if(USER) e2=result; l+=p2.p.r;
			    hist=p2.o.history;
			    num+=p2.o.gens-gen;
			    vertices+=p2.o.gens-gen;
			    if(result>0){
				result=propagateTo(BIG, result, nz, 3, p3, 0,
						   p2.p.t, p2.p.x, p2.p.y, p2.p.z, p2.p.theta, p2.p.phi);
				l+=p3.p.r;
			    }
			}
			System.out.println("TR "+gen+" ? "+type+" "+Output.f(x)+" "+Output.f(y)+" "+Output.f(z)+" "+
					   Output.f(th)+" "+Output.f(phi)+" "+Output.f(l*1.e-2)+" "+Output.f(e)+" "+Output.f(t));
			System.out.print(hist);
			if(USER) if(e>e0){
			    e0=e; er1=e1; er2=e2; erc=ec;
			    hr1=h1; hr2=h2; nrx=nx; nry=ny; nrz=nz;
			}
		    }
		    else{
			type="TR "+gen+" ? "+type+st.nextToken("\n");
			System.out.println(type);
		    }
		}
		else if("EM".equals(taux)){
		    events++;
		    num=0;
		    if(USER){
			e0=0; er1=0; er2=0; erc=0;
			hr1=0; hr2=0; nrx=0; nry=0; nrz=0;
		    }
		    System.out.println(line);
		}
		else if("EE".equals(taux)){
		    if(USER) System.out.println("US mpmcwj-energy "+
						Output.f(erc)+" "+Output.f(er1)+" "+Output.f(er2)+" "+
						Output.f(nrx)+" "+Output.f(nry)+" "+Output.f(nrz)+" "+
						Output.f(hr1)+" "+Output.f(hr2)+" "+Output.f(e0));
		    System.out.println(line);
		}
		else if("TBEGIN".equals(taux)){
		    if(USER) System.out.println("USER_DEF mpmcwj-energy E_CPD E_IN E_OUT CDP_X CDP_Y CDP_Z Z_IN Z_OUT E_INI");
		    System.out.println(line);
		}
		else System.out.println(line);
	    }
	    if(events!=0) vertices/=events;
	    System.err.println("events read "+events+" tracks looped "+tracks+
			       " average vertices "+vertices+" missed volume "+missed);
	}catch(Exception error){
	    System.err.println("Program finished with exception: "+error.toString());
	    throw new mmcException("input error");
	}
    }

    //----------------------------------------------------------------------------------------------------//

    private static int flag=0;
    private static double ec=0;

    /**
     * checks the flag, corrects the units, initializes the particle and calls the propagator
     */

    private static double propagateTo(double h, double e, double n, int i, Propagate p, int igen,
				      double time, double x, double y, double z, double theta, double phi){
	double result;
	switch(i){
	case 1: p.o.initDefault(type, time, x, y, z, theta, phi); break;
	case 2: p.o.initF2000(igen, type, time, x, y, z, theta, phi); break;
	case 3: p.o.initDefault(type, time, x, y, z, theta, phi); break;
	}
	if(flag==i){
	    result=p.propagateTo(h*1.e2, e*1.e3, n*1.e2);
	    ec=p.getPropEc();
	    ec=ec>0?ec*1.e-3:ec*1.e-2;
	}
	else result=p.propagateTo(h*1.e2, e*1.e3);
	return result>0?result*1.e-3:result*1.e-2;
    }

}
