import mmc.*;
import java.io.InputStreamReader;
import java.io.LineNumberReader;
import java.util.StringTokenizer;

public class Frejus{

    public static void main(String[] args){

	double vcut=0.01;
	String medium="Frejus Rock";
	boolean timef=false;
	boolean conti=false;

	for(int n=0; n<args.length; n++){
	    if(args[n].equals("-help")){
		System.out.println("\n"+
"This program propagates muons of given energy through given distance\n"+
"Available options are: -vcut=[value of vcut: 0<vcut<=1]\n"+
"                       -medi=[medium name]\n"+
"                       -time also report time of flight\n"+
"                       -cont enable continuous loss randomization\n");
		return;
	    }
	    else if(args[n].equals("-time")){
		timef=true;
	    }
	    else if(args[n].equals("-cont")){
		conti=true;
	    }
	    else if(args[n].startsWith("-vcut=")){
		try{
		    vcut=Double.parseDouble(args[n].substring(6));
		}catch(Exception error){
		    vcut=-1;
		}
	    }
	    else if(args[n].startsWith("-medi=")){
		try{
		    medium=args[n].substring(6);
		}catch(Exception error){
		    medium="";
		}
	    }
	}

	Propagate p = new Propagate(medium, -1, vcut);
	Output.DEBUG=false;
	System.err.println(Output.version);
	//p.p.low=4.05e2;        // Default settings for the Frejus run are: p.low=105+300, vcut=0.2
	//p.s.n.measured=true;   // Comment this out to disable parametrization of the photon-nucleon cross section
	//p.s.b.lorenzCut=1.e7;  p.s.b.lorenz=true;  // Enable Lorenz invariant violation - cutoff energy in [MeV]
	p.exactTime=timef;     // To ignore timing corrections of slow particles
	p.contiCorr=conti;     // To randomize the continuous energy losses distributed according to 1/e law
	p.interpolate("all", ".frejus");  // Comment this out to leave out parametrizations
	System.err.println("Enter the following: energy in [GeV] and distance to travel x in [m].");
	System.err.println("The last entry in the output is the final energy in [GeV] (if > 0)");
	System.err.println("or distance traveled to the point of disappearance in [m] (if < 0).");
	System.err.println("          ---  *** Enter your parameters now ***  ---");

	try{
	    LineNumberReader file = new LineNumberReader(new InputStreamReader(System.in));
	    String line;
	    StringTokenizer t;
	    double e, x, result;
	    while((line=file.readLine())!=null){
		t = new StringTokenizer(line);
		e=Double.parseDouble(t.nextToken());
		x=Double.parseDouble(t.nextToken());
		result=p.propagateTo(x*1.e2, e*1.e3);
		line+=" "+Output.f(result>0?result*1.e-3:result*1.e-2);
		if(timef) line+=" "+Output.f(p.p.t);
		System.out.println(line);
	    }
	}catch(Exception error){
	    System.err.println("Program finished with exception: "+error.toString());
	    throw new mmcException("input error");
	}
    }

}
