import mmc.*;

public class TestPropagate{

    private static double rnd1=0.5, rnd2=0.5;

    public static void main(String[] args){
	int test=1;  // 1 for check_crosS, 2 for check_proP
	String intr="";

	for(int n=0; n<args.length; n++){
	    if(args[n].equals("-help")){
		System.out.println("\n"+
"This program tests parametrizations used by mmc\n"+
"Available options are: -proc=[TEST to perform: 1 or 2]\n"+
"                       -intr=[interpolate: all, crs or \"\"]\n"+
"                       -RND1=[first random number]\n"+
"                       -RND2=[second random number]\n");
		return;
	    }
	    else if(args[n].startsWith("-proc=")){
		try{
		    test=(int)Double.parseDouble(args[n].substring(6));
		}catch(Exception error){
		    test=0;
		}
	    }
	    else if(args[n].startsWith("-intr=")){
		try{
		    intr=args[n].substring(6);
		}catch(Exception error){
		    intr="";
		}
	    }
	    else if(args[n].startsWith("-RND1=")){
		try{
		    rnd1=Double.parseDouble(args[n].substring(6));
		}catch(Exception error){
		    rnd1=0.5;
		}
	    }
	    else if(args[n].startsWith("-RND2=")){
		try{
		    rnd2=Double.parseDouble(args[n].substring(6));
		}catch(Exception error){
		    rnd2=0.5;
		}
	    }
	}

	Propagate p = new Propagate("Ice", -1., .01);
	System.out.println(Output.version);
	//p.s.n.measured=true;  // Comment this out to disable parametrization of the photon-nucleon cross section
	//p.s.b.lorenzCut=1.e7;  p.s.b.lorenz=true;  // Enable Lorenz invariant violation - cutoff energy in [MeV]
	p.interpolate(intr, ".test");  // Comment this out to leave out parametrizations
	Output.DEBUG=false;

	switch(test){
	case 1: check_crosS(p); break;
	case 2: check_proP(p); break;
	default: System.err.println("Error: the proc="+test+" is not defined");
	}
    }

    private static void check_crosS(Propagate p){
	double ind=rnd1;  // random number, chooses element
	double rnd=rnd2;  // .0, ... ,1., chooses lost energy
	double save, aux, sum, part, e;
	aux=1/(p.m.Ro*1.e3);
	for(e=120; e<1.e14; e*=1.2){
	    sum=0;
	    p.p.setEnergy(e);
	    System.out.println("Energy is "+Output.f(e/1.e3)+" GeV, vCut = "+Output.f(p.m.vCut(p.p.e))+
			       ", medium is "+p.m.name+", energy losses are in [GeV g-1 cm2]");

	    part=p.s.i.c.dEdx()*aux; sum+=part;
	    System.out.print("Ioniz: "+Output.f(part)+" \t "+Output.f(save=p.s.i.s.dNdx(rnd)));
	    if(save!=0) save=p.s.i.s.e(ind); System.out.println(" \t "+Output.f(save/e));

	    part=p.s.b.c.dEdx()*aux; sum+=part;
	    System.out.print("Brems: "+Output.f(part)+" \t "+Output.f(save=p.s.b.s.dNdx(rnd)));
	    if(save!=0) save=p.s.b.s.e(ind); System.out.println(" \t "+Output.f(save/e));

	    part=p.s.n.c.dEdx()*aux; sum+=part;
	    System.out.print("Photo: "+Output.f(part)+" \t "+Output.f(save=p.s.n.s.dNdx(rnd)));
	    if(save!=0) save=p.s.n.s.e(ind); System.out.println(" \t "+Output.f(save/e));

	    part=p.s.e.c.dEdx()*aux; sum+=part;
	    System.out.print("Epair: "+Output.f(part)+" \t "+Output.f(save=p.s.e.s.dNdx(rnd)));
	    if(save!=0) save=p.s.e.s.e(ind); System.out.println(" \t "+Output.f(save/e));

	    System.out.println("Total: "+Output.f(sum));
	}
    }

    private static void check_proP(Propagate p){
	double rnd=rnd1;  // random number, chooses final energy
	double xfd=rnd2;  // .0, ... ,1., chooses final distance
	double rndMin, ini, e, ef, xf;
	rnd=-Math.log(rnd);
	for(e=120; e<1.e14; e*=1.2){
	    System.out.println("Energy is "+Output.f(e/1.e3)+" GeV");
	    System.out.print(Output.f(rndMin=p.getpr(e, rnd))+" \t ");
	    if(rnd<rndMin) System.out.print(Output.f(ef=p.getef(e, rndMin, rnd))+" \t ");
	    else{ ef=p.p.low; System.out.print(0+" \t "); }
	    ini=p.s.getini(e); xf=p.s.getdx(e, ef, ini, 0)*xfd;
	    System.out.print(Output.f(p.s.getdx(e, ef, ini, xf))+" \t ");
	    System.out.println(Output.f(p.s.getef(e, ini, xf)));
	}
    }

}
