package mmc;

/**
 * class contains functions necessary for calculation of bremsstrahlung losses
 */

public class Bremsstrahlung extends CrossSections{

    public BremsContinuous c;
    public BremsStochastic s;

    protected double vMax, vUp;
    protected Bremsstrahlung brems;

    //----------------------------------------------------------------------------------------------------//

    /**
     * creates internal references to p and m, to be called from subclasses
     */

    public Bremsstrahlung(Bremsstrahlung cros){
	p=cros.p;
	m=cros.m;
	this.cros=cros.cros;
	brems=cros;
    }

    //----------------------------------------------------------------------------------------------------//

    /**
     * initializes subclasses and creates internal references to p and m
     */

    public Bremsstrahlung(CrossSections cros){
	super(cros);
	brems=this;
	c = new BremsContinuous(this);
	s = new BremsStochastic(this);
    }

    //----------------------------------------------------------------------------------------------------//

    /**
     * call before using the bremsstrahlung functions to set the component of the primary
     */

    protected void setEnergy(int i){
	cros.component=i;
	vMax=1-(3./4)*sqrtE*(p.m/p.e)*Math.pow(m.Z[i], 1./3);
	if(vMax<0) vMax=0;
	if(brems.lorenz) vMax=Math.min(vMax, brems.lorenzCut/p.e);
	vMax=Math.min(vMax, 1-p.m/p.e);
	vUp=Math.min(vMax, m.vCut(p.e));
    }

    //----------------------------------------------------------------------------------------------------//

    /**
     * this is what the Elastic Bremsstrahlung Cross Section (EBCS) is equal to
     * units are [1/cm] since the multiplication by No*n is done here.
     * Corrections for exitations of the nucleus and deep inelastic exitations of separate nucleons are
     * included (positive term dn/Z), as well as the contribution of the mu-diagrams to the inelastic
     * bremsstrahlung on the electrons (non-zero only for allowed energies of photon after electron recoil).
     */

    protected double Sel(double v, int i){
	double d, da, dn, Dn, Fa, maxV, aux, step;
	d=p.m*p.m*v/(2*p.e*(1-v));
	da=Math.log(1+Me/(d*sqrtE*m.B[i]*Math.pow(m.Z[i], -1./3)));
	Dn=1.54*Math.pow(m.A[i], 0.27);
	dn=Math.log(Dn/(1+d*(Dn*sqrtE-2)/p.m));
	maxV=Me*(p.e-p.m)/(p.e*(p.e-p.p+Me));
	if(v<maxV) Fa=Math.log((p.m/d)/(d*p.m/(Me*Me)+sqrtE))-Math.log(1+Me/(d*sqrtE*m.P[i]*Math.pow(m.Z[i], -2./3)));
	else Fa=0;
	if(m.Z[i]==1) step=0;
	else step=1;
	aux=2*m.Z[i]*(Me/p.m)*Re;
	aux*=(Alpha/v)*aux*((4./3)*(1-v)+v*v)*(Math.log(p.m/d)-0.5-da-dn+(dn*step+Fa)/m.Z[i]);
	return m.No*m.n[i]*aux;
    }

    public boolean lorenz=false;
    public double lorenzCut=1.e6;  // in [MeV]

}
