package mmc;

/**
 * initializes all cross sections and keeps references to them
 */

public class CrossSections extends PhysicsModel{

    public Decay d;
    public Ionizationloss i;
    public Bremsstrahlung b;
    public Photonuclear n;
    public Epairproduction e;
    public int component=0;

    protected Particle p;
    protected Medium m;
    protected CrossSections cros;

    private Integral I;

    //----------------------------------------------------------------------------------------------------//

    /**
     * Necessary to keep class structure, since it is called from subclasses
     */

    public CrossSections(){

    }

    //----------------------------------------------------------------------------------------------------//

    /**
     * creates internal references to p and m, to be called from subclasses
     */

    public CrossSections(CrossSections cros){
	p=cros.p;
	m=cros.m;
	this.cros=cros;
    }

    //----------------------------------------------------------------------------------------------------//

    /**
     * initializes all cross sections, creates internal references to p and m
     */

    public CrossSections(Particle p, Medium m){
	this.p=p;
	this.m=m;
	d = new Decay(this);
	i = new Ionizationloss(this);
	b = new Bremsstrahlung(this);
	n = new Photonuclear(this);
	e = new Epairproduction(this);
 	I = new Integral(5, 40, 1.e-5);
    }

    //----------------------------------------------------------------------------------------------------//

    /**
     * function for range calculation for given energy - interface to Integral
     */

    public double function(double E){
	final boolean DEBUG=false;
	double result, aux;
	p.setEnergy(E);
	result=0;
	if(DEBUG) System.err.print(" * "+Output.f(p.e));
	aux=i.c.dEdx();
	result+=aux;
	if(DEBUG) System.err.print(" \t "+Output.f(aux));
	aux=b.c.dEdx();
	result+=aux;
	if(DEBUG) System.err.print(" \t "+Output.f(aux));
	aux=n.c.dEdx();
	result+=aux;
	if(DEBUG) System.err.print(" \t "+Output.f(aux));
	aux=e.c.dEdx();
	result+=aux;
	if(DEBUG) System.err.println(" \t "+Output.f(aux));
	return -1/result;
    }

    //----------------------------------------------------------------------------------------------------//

    /**
     * initialize get functions for given initial energy ei
     */

    public double getini(double ei){
	if(jt) return J.interpolate(ei);
	else return 0;
    }

    //----------------------------------------------------------------------------------------------------//

    /**
     * returns the value of the distance integral
     */

    public double getdx(double ei, double ef, double ini, double dist){
	if(jt) return Math.max(ini-J.interpolate(ef), 0);
	else return I.integrateWithLog(ei, ef, this, -dist);
    }

    //----------------------------------------------------------------------------------------------------//

    /**
     * final displacement, corresponding to the given value dist of the distance integral
     */

    public double getef(double ei, double ini, double dist){
	if(jt) return Math.min(Math.max(J.findLimit(ini-dist), p.low), ei);
	else return I.getUpperLimit();
    }

    //----------------------------------------------------------------------------------------------------//

    public Interpolate J;
    public boolean jt=false;

    /**
     * 1d parametrization - interface to Interpolate
     */

    public double functionInt(double e){
	return I.integrateWithLog(e, p.low, this);
    }

}
