package mmc;

/**
 * class contains functions necessary for calculation of e+e- pair production losses
 */

public class Epairproduction extends CrossSections{

    public EpairContinuous c;
    public EpairStochastic s;

    protected double vMax, vUp, vMin;
    protected Epairproduction epair;

    private Integral I;
    private int i;
    private double v;

    //----------------------------------------------------------------------------------------------------//

    /**
     * creates internal references to p and m, to be called from subclasses
     */

    public Epairproduction(Epairproduction cros){
	p=cros.p;
	m=cros.m;
	this.cros=cros.cros;
	epair=cros;
    }

    //----------------------------------------------------------------------------------------------------//

    /**
     * initializes subclasses and creates internal references to p and m
     */

    public Epairproduction(CrossSections cros){
	super(cros);
	epair=this;
	c = new EpairContinuous(this);
	s = new EpairStochastic(this);
	I = new Integral(5, 40, 1.e-6);
    }

    //----------------------------------------------------------------------------------------------------//

    /**
     * call before using the pair production functions to set the component of the primary
     */

    public void setEnergy(int i){
	double aux;
	cros.component=i;
	vMin=4*Me/p.e;
	vMax=1-(3./4)*sqrtE*(p.m/p.e)*Math.pow(m.Z[i], 1./3);
	aux=p.m/p.e;
	aux=1-6*aux*aux;
	vMax=Math.min(vMax, aux);
	//if(p.e<1.e4) vMax=vMin;
	vMax=Math.min(vMax, 1-p.m/p.e);
	if(vMax<vMin) vMax=vMin;
	vUp=Math.min(vMax, m.vCut(p.e));
	if(vUp<vMin) vUp=vMin;
    }

     //----------------------------------------------------------------------------------------------------//

    /**
     * this is the calculation of the d2Sigma/dvdRo - interface to Integral
     */

    public double function(double r){
	double Fe, Fm, Le, Lm, Ye, Ym, s, b, k, g1, g2;
	double aux, aux1, aux2, r2, Z3;
	r=1-r; // only for integral optimization - do not forget to swap integration limits!
	r2=r*r;
	Z3=Math.pow(m.Z[i], -1./3);
	aux=(p.m*v)/(2*Me);
	aux*=aux;
	s=aux*(1-r2)/(1-v);
	b=(v*v)/(2*(1-v));
	Ye=(5-r2+4*b*(1+r2))/(2*(1+3*b)*Math.log(3+1/s)-r2-2*b*(2-r2));
	Ym=(4+r2+3*b*(1+r2))/((1+r2)*(1.5+2*b)*Math.log(3+s)+1-1.5*r2);

	aux=(1.5*Me)/(p.m*Z3);
	aux*=aux;
	aux1=(1+s)*(1+Ye);
	aux2=(2*Me*sqrtE*m.B[i]*Z3)/(p.e*v*(1-r2));
	Le=Math.log((m.B[i]*Z3*Math.sqrt(aux1))/(1+aux2*aux1))-0.5*Math.log(1+aux*aux1);
	Lm=Math.log(((p.m/(1.5*Me))*m.B[i]*Z3*Z3)/(1+aux2*(1+s)*(1+Ym)));

	Fe=(((2+r2)*(1+b)+s*(3+r2))*Math.log(1+1/s)+(1-r2-b)/(1+s)-(3+r2))*Le;
	Fm=(((1+r2)*(1+1.5*b)-(1+2*b)*(1-r2)/s)*Math.log(1+s)+s*(1-r2-b)/(1+s)+(1+2*b)*(1-r2))*Lm;

	if(m.Z[i]==1){ g1=4.4e-5; g2=4.8e-5; }
	else{ g1=1.95e-5; g2=5.3e-5; }
	aux=p.e/p.m;
	k=0.058*Math.log(aux/(1+g1*aux/Z3))-0.14;
	if(k<=0) k=0;
	else k=(0.073*Math.log(aux/(1+g2*aux/(Z3*Z3)))-0.26)/k;
	if(k<0) k=0;

	aux=Alpha*Re;
	aux*=aux/(1.5*Pi);
	aux1=Me/p.m;
	aux1*=aux1;

	aux*=2*m.Z[i]*(m.Z[i]+k)*((1-v)/v)*(Fe+aux1*Fm);
	if(aux<0) aux=0;
	return aux;
    }

    //----------------------------------------------------------------------------------------------------//

    /**
     * this is the calculation of the dSigma/dv
     */

    public double ePair(double v, int i){
	if(jt){
	    setEnergy(cros.component);
	    if(v>=vUp) return Math.max(J[i].interpolate(p.e, Math.log(v/vUp)/Math.log(vMax/vUp)), 0);
	}
	double rMax, aux, aux2;
	this.i=i;
	this.v=v;
	aux=1-(4*Me)/(p.e*v);
	aux2=1-(6*p.m*p.m)/(p.e*p.e*(1-v));
	if(aux>0 && aux2>0) rMax=Math.sqrt(aux)*aux2;
	else rMax=0;
	return m.No*m.n[i]*I.integrateWithLog(1-rMax, 1, this);
    }

    //----------------------------------------------------------------------------------------------------//

    public Interpolate J[];
    public boolean jt=false;

    /**
     * 2d parametrization - interface to Interpolate
     */

    public double functionInt(double e, double v){
	p.setEnergy(e);
	setEnergy(cros.component);
	if(vUp==vMax) return 0;
	v=vUp*Math.exp(v*Math.log(vMax/vUp));
	return ePair(v, cros.component);
    }

}
