package mmc;

/**
 * This is an entry class of the physical model we are building. Contains constant definitions.
 */

public class PhysicsModel implements FunctionOfx, FunctionInt, FunctionInt2{

    final protected static double Pi=Math.PI;               // 3.14159...
    final protected static double Log10=Math.log(10);       // log(10)
    final protected static double sqrtE=Math.sqrt(Math.E);  // sqrt(e)
    final protected static double computerPrecision=1.e-10; // 1.e-11 may also work

    final protected static double Alpha=1/137.03599976;     // fine structure constant
    final protected static double Me=0.510998902;           // electron mass (MeV)
    final protected static double K=0.307075;               // in the ionization formula (MeV*cm2/g)

    final protected static double C=299792458e2;            // spead of light (cm/s)
    final protected static double Re=2.817940285e-13;       // classical electron radius (cm)
    final protected static double Na=6.02214199e23;         // Avogadro's number (1/mol)

    final protected static double Mpi=139.57018;            // pion mass (MeV)
    final protected static double Mp=938.271998;            // proton mass (MeV)
    final protected static double Mn=939.56533;             // neutron mass (MeV)

    //----------------------------------------------------------------------------------------------------//

    /**
     * for integration - interface to Integral - to be overwritten by subclasses
     */

    public double function(double e){
	return 0;
    }

    //----------------------------------------------------------------------------------------------------//

    /**
     * 1d parametrization - interface to Interpolate - to be overwritten by subclasses
     */

    public double functionInt(double e){
	return 0;
    }

    //----------------------------------------------------------------------------------------------------//

    /**
     * 2d parametrization - interface to Interpolate - to be overwritten by subclasses
     */

    public double functionInt(double e, double v){
	return 0;
    }

}
